/* 
 * Copyright (C) 2004-2005 Jonathan Bindel
 * Copyright (C) 2006-2007 Eskil Bylund
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

using System;
using System.Collections.Generic;

namespace DCSharp.Backend.Objects
{
	public class SearchEventArgs : EventArgs
	{
		public SearchEventArgs(SearchInfo searchInfo)
		{
			this.searchInfo = searchInfo;
		}
		
		private SearchInfo searchInfo;
		
		public SearchInfo SearchInfo
		{
			get
			{
				return searchInfo;
			}
		}
	}

	public enum SearchFileType
	{
		Any = 1,
		Audio,
		Package,
		Document,
		Executable,
		Image,
		Video,
		Directory
	}

	public class SearchInfo
	{
		public event EventHandler<SearchResultEventArgs> SearchResultAdded;

		public SearchInfo(string[] keywords, SearchFileType type, long? maxSize,
			long? minSize) : this()
		{
			this.keywords = keywords;
			this.type = type;
			this.maxSize = maxSize;
			this.minSize = minSize;
		}

		public SearchInfo(string tth) : this()
		{
			this.tth = tth;
		}

		protected SearchInfo()
		{
			searchResults = new List<SearchResult>();
		}

		#region Properties

		private string[] keywords;

		public string[] Keywords
		{
			get
			{
				return keywords;
			}
		}

		private SearchFileType type;

		public SearchFileType Type
		{
			get
			{
				return type;
			}
		}

		private long? maxSize;

		public long? MaxSize
		{
			get
			{
				return maxSize;
			}
		}

		private long? minSize;

		public long? MinSize
		{
			get
			{
				return minSize;
			}
		}

		private string tth;

		public string TTH
		{
			get
			{
				return tth;
			}
		}

		private List<SearchResult> searchResults;

		public SearchResult[] SearchResults
		{
			get
			{
				return searchResults.ToArray();
			}
		}

		#endregion

		#region Methods

		public void Add(SearchResult result)
		{
			if (result == null)
			{
				throw new ArgumentNullException("result");
			}
			
			if (!searchResults.Contains(result))
			{
				searchResults.Add(result);
				OnSearchResultAdded(result);
			}
		}

		protected virtual void OnSearchResultAdded(SearchResult result)
		{
			if (SearchResultAdded != null)
			{
				SearchResultAdded(this, new SearchResultEventArgs(result));
			}
		}

		#endregion
	}
}
